@@ -472,6 +472,8 @@ gboolean          gst_omx_port_ensure_buffer_count_actual (GstOMXPort * port, gu
 gboolean          gst_omx_port_update_buffer_count_actual (GstOMXPort * port, guint nb);
 
 gboolean          gst_omx_port_set_dmabuf (GstOMXPort * port, gboolean dmabuf);
+gboolean          gst_omx_port_set_subframe (GstOMXPort * port, gboolean enabled);
+gboolean          gst_omx_port_get_subframe (GstOMXPort * port);
 
 /* OMX 1.2.0 dynamic allocation mode */
 gboolean          gst_omx_is_dynamic_allocation_supported (void);
@@ -483,9 +485,6 @@ gboolean          gst_omx_buffer_import_fd (GstOMXBuffer * buffer, GstBuffer * i
 
 void              gst_omx_set_default_role (GstOMXClassData *class_data, const gchar *default_role);
 
-void              gst_omx_buffer_set_omx_buf (GstBuffer * buffer, GstOMXBuffer * omx_buf);
-GstOMXBuffer *    gst_omx_buffer_get_omx_buf (GstBuffer * buffer);
-
 /* refered by plugin_init */
 GST_DEBUG_CATEGORY_EXTERN (gst_omx_video_debug_category);
 
